﻿
// Classe matrix4X4 (V1).
// F Heintz (www.fredHeintz.com)

// Operations on 4 by 4 components Matrix

class _engine3DAS.matrix4X4
{
public var	_mtrx:Array;

private var	_toRadFactor:Number;
	
	// ----------------------------------------------------------------------------------------------------------------
	// Constructor
	public function matrix4X4()
	{
		_toRadFactor = (Math.PI / 180.0);
		// Bidimentional Array
		_mtrx = new Array();
		for(var i:Number = 0; i < 4; i++)
		{
			_mtrx[i] = new Array();
		}
	}
	
	// Initialize the matrix with the identity matrix.
	public function setIdentity():Void
	{
		_mtrx[0][0] = 1.0;
		_mtrx[1][0] = 0.0;
		_mtrx[2][0] = 0.0;
		_mtrx[3][0] = 0.0;
		
		_mtrx[0][1] = 0.0;
		_mtrx[1][1] = 1.0;
		_mtrx[2][1] = 0.0;
		_mtrx[3][1] = 0.0;
		
		_mtrx[0][2] = 0.0;
		_mtrx[1][2] = 0.0;
		_mtrx[2][2] = 1.0;
		_mtrx[3][2] = 0.0;
		
		_mtrx[0][3] = 0.0;
		_mtrx[1][3] = 0.0;
		_mtrx[2][3] = 0.0;
		_mtrx[3][3] = 1.0;
	}
	
	// Initialize the matrix with a rotation on the X axis.
	public function setRotX(angleX:Number):Void
	{
		setIdentity();
		var angleRad:Number = angleX * _toRadFactor;
		_mtrx[1][1] = Math.cos(angleRad);
		_mtrx[1][2] = Math.sin(angleRad);
		_mtrx[2][1] = -_mtrx[1][2];
		_mtrx[2][2] = _mtrx[1][1];
	}
	
	// Initialize the matrix with a rotation on the Y axis.
	public function setRotY(angleY:Number):Void
	{
		setIdentity();
		var angleRad:Number = angleY * _toRadFactor;
		_mtrx[0][0] = Math.cos(angleY * Math.PI / 180.0);
		_mtrx[2][0] = Math.sin(angleY * Math.PI / 180.0);
		_mtrx[0][2] = -_mtrx[2][0];
		_mtrx[2][2] = _mtrx[0][0];
	}
	
	// Initialize the matrix with a rotation on the Z axis.
	public function setRotZ(angleZ:Number):Void
	{
		setIdentity();
		var angleRad:Number = angleZ * _toRadFactor;
		_mtrx[0][0] = Math.cos(angleZ * Math.PI / 180.0);
		_mtrx[0][1] = Math.sin(angleZ * Math.PI / 180.0);
		_mtrx[1][0] = -_mtrx[0][1];
		_mtrx[1][1] = _mtrx[0][0];
	}
	
	// Initialize the world Matrix from 3 Angles.
	public function setFromAngles(angleX:Number, angleY:Number, angleZ:Number):Void
	{
		// MX = rotation on X
		var	matrixTmpX:matrix4X4 = new matrix4X4;
		matrixTmpX.setRotX(angleX);
		
		// MY = rotation on Y
		var	matrixTmpY:matrix4X4 = new matrix4X4;
		matrixTmpY.setRotY(angleY);
		
		// MZ = rotation on Z
		var	matrixTmpZ:matrix4X4 = new matrix4X4;
		matrixTmpZ.setRotZ(angleZ);
		
		// Build the new Matrix (MX x MY x MZ)
		mtrxCopy(matrixTmpX);
		mtrxMultiply(matrixTmpY);
		mtrxMultiply(matrixTmpZ);
	}
	
	// Initialize a lookAt type Matrix.
	public function setLookAt(xView:Number, yView:Number, zView:Number, xTgt:Number, yTgt:Number, zTgt:Number, xUp:Number, yUp:Number, zUp:Number):Void
	{
		setIdentity();
		// 1) Build the Z vector.
		var zVct_X:Number = xTgt - xView;
		var zVct_Y:Number = yTgt - yView;
		var zVct_Z:Number = zTgt - zView;
		// Normalize (set to unit vector).
		var size:Number = Math.sqrt((zVct_X * zVct_X) + (zVct_Y * zVct_Y) + (zVct_Z * zVct_Z));
		zVct_X /= size;
		zVct_Y /= size;
		zVct_Z /= size;		
		
		// 2) Build the Right vector rVct = upVct cross zVct
		var rVct_X:Number = (yUp * zVct_Z) - (zVct_Y * zUp);
		var rVct_Y:Number = (zUp * zVct_X) - (zVct_Z * xUp);
		var rVct_Z:Number = (xUp * zVct_Y) - (zVct_X * yUp);
		// normalize.
		size = Math.sqrt((rVct_X * rVct_X) + (rVct_Y * rVct_Y) + (rVct_Z * rVct_Z));
		rVct_X /= size;
		rVct_Y /= size;
		rVct_Z /= size;		
		
		// 3) Build the Up vector uVct = zVct cross rVct.
		var uVct_X:Number = (zVct_Y * rVct_Z) - (rVct_Y * zVct_Z);
		var uVct_Y:Number = (zVct_Z * rVct_X) - (rVct_Z * zVct_X);
		var uVct_Z:Number = (zVct_X * rVct_Y) - (rVct_X * zVct_Y);		
		// normalize.
		size = Math.sqrt((uVct_X * uVct_X) + (uVct_Y * uVct_Y) + (uVct_Z * uVct_Z));
		uVct_X /= size;
		uVct_Y /= size;
		uVct_Z /= size;		
		
		// 4) Build matrix.
		_mtrx[0][0] = rVct_X;
		_mtrx[1][0] = rVct_Y;
		_mtrx[2][0] = rVct_Z;
		_mtrx[3][0] = -((xView * rVct_X) + (yView * rVct_Y) + (zView * rVct_Z));
		_mtrx[0][1] = uVct_X;
		_mtrx[1][1] = uVct_Y;
		_mtrx[2][1] = uVct_Z;
		_mtrx[3][1] = -((xView * uVct_X) + (yView * uVct_Y) + (zView * uVct_Z));
		_mtrx[0][2] = zVct_X;
		_mtrx[1][2] = zVct_Y;
		_mtrx[2][2] = zVct_Z;
		_mtrx[3][2] = -((xView * zVct_X) + (yView * zVct_Y) + (zView * zVct_Z));
	}

	// Initialize our matrix with another.
	public function mtrxCopy(mtrxObj:matrix4X4):Void
	{
		for(var i:Number = 0; i < 4; i++)
		{
			for(var j:Number = 0; j < 4; j++)
			{
				_mtrx[i][j] = mtrxObj._mtrx[i][j];
			}
		}
	}
	
	// Multiply-Right our matrix with another.
	public function mtrxMultiply(mtrxObj:matrix4X4):Void
	{
		var m00:Number = _mtrx[0][0];
		var m10:Number = _mtrx[1][0];
		var m20:Number = _mtrx[2][0];
		var m30:Number = _mtrx[3][0];
		var m01:Number = _mtrx[0][1];
		var m11:Number = _mtrx[1][1];
		var m21:Number = _mtrx[2][1];
		var m31:Number = _mtrx[3][1];		
		var m02:Number = _mtrx[0][2];
		var m12:Number = _mtrx[1][2];
		var m22:Number = _mtrx[2][2];
		var m32:Number = _mtrx[3][2];		
		var m03:Number = _mtrx[0][3];
		var m13:Number = _mtrx[1][3];
		var m23:Number = _mtrx[2][3];
		var m33:Number = _mtrx[3][3];
		
		// Build matrix.
		_mtrx[0][0] = (m00 * mtrxObj._mtrx[0][0]) + (m10 * mtrxObj._mtrx[0][1]) + (m20 * mtrxObj._mtrx[0][2]) + (m30 * mtrxObj._mtrx[0][3]);
		_mtrx[1][0] = (m00 * mtrxObj._mtrx[1][0]) + (m10 * mtrxObj._mtrx[1][1]) + (m20 * mtrxObj._mtrx[1][2]) + (m30 * mtrxObj._mtrx[1][3]);
		_mtrx[2][0] = (m00 * mtrxObj._mtrx[2][0]) + (m10 * mtrxObj._mtrx[2][1]) + (m20 * mtrxObj._mtrx[2][2]) + (m30 * mtrxObj._mtrx[2][3]);
		_mtrx[3][0] = (m00 * mtrxObj._mtrx[3][0]) + (m10 * mtrxObj._mtrx[3][1]) + (m20 * mtrxObj._mtrx[3][2]) + (m30 * mtrxObj._mtrx[3][3]);

		_mtrx[0][1] = (m01 * mtrxObj._mtrx[0][0]) + (m11 * mtrxObj._mtrx[0][1]) + (m21 * mtrxObj._mtrx[0][2]) + (m31 * mtrxObj._mtrx[0][3]);
		_mtrx[1][1] = (m01 * mtrxObj._mtrx[1][0]) + (m11 * mtrxObj._mtrx[1][1]) + (m21 * mtrxObj._mtrx[1][2]) + (m31 * mtrxObj._mtrx[1][3]);
		_mtrx[2][1] = (m01 * mtrxObj._mtrx[2][0]) + (m11 * mtrxObj._mtrx[2][1]) + (m21 * mtrxObj._mtrx[2][2]) + (m31 * mtrxObj._mtrx[2][3]);
		_mtrx[3][1] = (m01 * mtrxObj._mtrx[3][0]) + (m11 * mtrxObj._mtrx[3][1]) + (m21 * mtrxObj._mtrx[3][2]) + (m31 * mtrxObj._mtrx[3][3]);

		_mtrx[0][2] = (m02 * mtrxObj._mtrx[0][0]) + (m12 * mtrxObj._mtrx[0][1]) + (m22 * mtrxObj._mtrx[0][2]) + (m32 * mtrxObj._mtrx[0][3]);
		_mtrx[1][2] = (m02 * mtrxObj._mtrx[1][0]) + (m12 * mtrxObj._mtrx[1][1]) + (m22 * mtrxObj._mtrx[1][2]) + (m32 * mtrxObj._mtrx[1][3]);
		_mtrx[2][2] = (m02 * mtrxObj._mtrx[2][0]) + (m12 * mtrxObj._mtrx[2][1]) + (m22 * mtrxObj._mtrx[2][2]) + (m32 * mtrxObj._mtrx[2][3]);
		_mtrx[3][2] = (m02 * mtrxObj._mtrx[3][0]) + (m12 * mtrxObj._mtrx[3][1]) + (m22 * mtrxObj._mtrx[3][2]) + (m32 * mtrxObj._mtrx[3][3]);

		_mtrx[0][3] = (m03 * mtrxObj._mtrx[0][0]) + (m13 * mtrxObj._mtrx[0][1]) + (m23 * mtrxObj._mtrx[0][2]) + (m33 * mtrxObj._mtrx[0][3]);
		_mtrx[1][3] = (m03 * mtrxObj._mtrx[1][0]) + (m13 * mtrxObj._mtrx[1][1]) + (m23 * mtrxObj._mtrx[1][2]) + (m33 * mtrxObj._mtrx[1][3]);
		_mtrx[2][3] = (m03 * mtrxObj._mtrx[2][0]) + (m13 * mtrxObj._mtrx[2][1]) + (m23 * mtrxObj._mtrx[2][2]) + (m33 * mtrxObj._mtrx[2][3]);
		_mtrx[3][3] = (m03 * mtrxObj._mtrx[3][0]) + (m13 * mtrxObj._mtrx[3][1]) + (m23 * mtrxObj._mtrx[3][2]) + (m33 * mtrxObj._mtrx[3][3]);
	}
}















