﻿
// Classe mesh (V1).
// F Heintz (www.fredHeintz.com)

// Stores components of a Mesh.

import _engine3DAS.vertex
import _engine3DAS.polygon
import _engine3DAS.meshLoader

class _engine3DAS.mesh
{
public var _nbVertexs:Number;
public var _vertexList:Array;
public var _nbPolygons:Number;
public var _polygonList:Array;

private var  _meshLoader:meshLoader;
	
	// ----------------------------------------------------------------------------------------------------------------
	// Constructor
	public function mesh(nbVertexs:Number, nbPolygons:Number)
	{
		// Allocate the vertexs
		initVertexList(nbVertexs);
		// Allocate the polygons
		initPolygonList(nbPolygons);
		// The Mesh loader
		_meshLoader = null;
	}

	// Initialize the Vertex List
	public function initVertexList(amount:Number):Void
	{
		_nbVertexs = amount;
		_vertexList = new Array();
		for(var i:Number = 0; i < _nbVertexs; i++)
		{
			_vertexList[i] = new vertex();
		}		
	}
	
	// Initialize the Polygon List
	public function initPolygonList(amount:Number):Void
	{
		_nbPolygons = amount;
		_polygonList = new Array();
		for(var i:Number = 0; i < _nbPolygons; i++)
		{
			_polygonList[i] = new polygon();
		}		
	}
	
	// Load a mesh.
	public function loadXml(fileName:String):Void
	{
		if(_meshLoader == null)
		{
			_meshLoader = new meshLoader();
		}
		_meshLoader.loadXml(fileName, this);
	}
	
	// Loading finished ?.
	public function isLoaded():Boolean
	{
		return(_meshLoader._bLoaded);
	}
	
	// Initialize one Vertex of the Mesh.
	public function setVertex(vertexIndex:Number, x3d:Number, y3d:Number, z3d:Number):Void
	{
		var vertexObj:vertex = _vertexList[vertexIndex];
		vertexObj.setCoords(x3d, y3d, z3d);
	}
	
	// Initialize one Polygon of the Mesh.
	public function setPolygon(polygonIndex:Number, color:Number, vert1:Number, vert2:Number, vert3:Number, vert4:Number, nx:Number, ny:Number, nz:Number):Void
	{
		var polygonObj:polygon = _polygonList[polygonIndex];
		polygonObj.addVertex(vert1);
		polygonObj.addVertex(vert2);
		polygonObj.addVertex(vert3);
		if(vert4 >= 0)
		{
			polygonObj.addVertex(vert4);
		}
		polygonObj.setNormal(nx, ny, nz);
		polygonObj.setColor((color >> 16)&0x0ff, (color >> 8)&0x0ff, color&0x0ff);
	}
	
	// Compute polygons visibility.
	public function cullMesh():Void
	{
	var	Q:Number;
	var	dx1:Number;
	var	dy1:Number;
	var	dx2:Number;
	var	dy2:Number;
	var	vert1:Number;
	var	vert2:Number;
	var	vert3:Number;
	var	vertexObj1:vertex;
	var	vertexObj2:vertex;
	var	vertexObj3:vertex;
	var	polygonObj:polygon;
	
		for(var i:Number = 0; i < _nbPolygons; i++)
		{
			polygonObj = _polygonList[i];
			if(polygonObj._nbIndexs < 3)
			{
				polygonObj._bVisible = false;
				continue;
			}
			vert1 = polygonObj._indexList[0];
			vert2 = polygonObj._indexList[1];
			vert3 = polygonObj._indexList[2];
			vertexObj1 = _vertexList[vert1];
			vertexObj2 = _vertexList[vert2];
			vertexObj3 = _vertexList[vert3];
			dx1 = (vertexObj1._x2D - vertexObj2._x2D);
			dy1 = (vertexObj1._y2D - vertexObj2._y2D);
			dx2 = (vertexObj3._x2D - vertexObj2._x2D);
			dy2 = (vertexObj3._y2D - vertexObj2._y2D);
			Q = ((dx1 * dy2) - (dx2 * dy1));
			if(Q >= 0.0)
			{
				polygonObj._bVisible = false;
				continue;
			}
			polygonObj._bVisible = true;
		}		
	}	
}



















