﻿
// Classe meshLoader (V1).
// F Heintz (www.fredHeintz.com)

// Load a mesh from an XML file.

import _engine3DAS.mesh
import _engine3DAS.vertex
import _engine3DAS.polygon

class _engine3DAS.meshLoader
{
public var	_bLoaded:Boolean;
public var _fileName:String;

private var	_materials:Array;
private var	_xmlObj:XML;
private var	_meshObj:mesh;
private var	_bDispXml:Boolean;

	// ----------------------------------------------------------------------------------------------------------------
	// Constructor
	public function meshLoader()
	{
		_bDispXml = true;
		_bLoaded = false;
		_xmlObj = new XML();
		_xmlObj.ignoreWhite = true;
	}
	
	// Load a mesh from a XML file.
	public function loadXml(fileName:String, meshObj:mesh):Void
	{
		trace("loadXml(" + fileName + ")");
		_bLoaded = false;
		_fileName = fileName;
		_meshObj = meshObj;

		var thisObj:meshLoader = this;
		_xmlObj.onLoad = function(bSuccess:Boolean)
		{
			trace("myOnLoad1(" + bSuccess + ")");		
			thisObj.myOnLoad(bSuccess);
		}
		_xmlObj.load(fileName);
	}
	
	// The Mesh has benn loaded
	public function xmlLoaded(bSuccess:Boolean, errorString:String):Void
	{
		_bLoaded = bSuccess;
		trace("xmlLoaded(" + bSuccess + ") <" + errorString + ">");		
	}
	
	// Callback XML on load
	public function myOnLoad(bSuccess:Boolean):Void
	{
		if(bSuccess)
		{
			if(_xmlObj.hasChildNodes() == true)
			{
				var curNode:XMLNode = getXmlNode(_xmlObj.firstChild, "FLASH3D_SCENE");
				if(curNode != null)
				{
					if(extractScene(curNode) == true)
					{
						xmlLoaded(true, "");
					}
					else
					{
						xmlLoaded(false, "fichier <" + _fileName + "> invalide");
					}
					return;
				}
			}
		}
		xmlLoaded(false, "fichier <" + _fileName + "> <" + _xmlObj.status + ">");
	}
	
	// Look for the main Node.
	public function getXmlNode(baseNode:XMLNode, leTitre:String):XMLNode
	{
		var curNode:XMLNode = baseNode;
		while(curNode)
		{
			if(curNode.nodeName == leTitre)
			{
				return(curNode);
			}
			curNode = curNode.nextSibling;
		}
		return(null);
	}
	
	// Read the Scene File.
	private function extractScene(leNode:XMLNode):Boolean
	{
		var curNode:XMLNode = null;
		var bOk:Boolean = false;
		if(leNode.hasChildNodes())
		{
			curNode = leNode.firstChild;
			while(curNode)
			{
				if(curNode.nodeName == "MATERIALS")
				{
					extractMaterials(curNode, curNode.attributes.amount);
				}
				if(curNode.nodeName == "OBJECT_FLAT")
				{
					extractMesh(curNode);
				}
				else if(curNode.nodeName == "FIN")
				{
					bOk = true;
				}
				curNode = curNode.nextSibling;
			}
		}
		if(_bDispXml == true)
		{
			trace("---------------------------------------");
			trace("nb Vertexs = " + _meshObj._nbVertexs);
			trace("nb Polygons = " + _meshObj._nbPolygons);
			trace("---------------------------------------");
		}
		return(bOk);
	}

	//
	private function extractMaterials(leNode:XMLNode, amount:Number):Void
	{
	var	a:Number;
	var	r:Number;
	var	g:Number;
	var	b:Number;
	
		if(_bDispXml == true)
		{
			trace("extractMaterials(" + amount + ")");
		}
		_materials = new Array();
		var materialPtr:Number = 0;
		var curNode:XMLNode = null;
		if(leNode.hasChildNodes())
		{
			curNode = leNode.firstChild;
			while(curNode)
			{
				if(curNode.nodeName == "MATERIAL")
				{
					a = parseInt(curNode.attributes.a);
					r = parseInt(curNode.attributes.r);
					g = parseInt(curNode.attributes.g);
					b = parseInt(curNode.attributes.b);
					_materials[materialPtr] = new Object();
					_materials[materialPtr].a = a;
					_materials[materialPtr].r = r;
					_materials[materialPtr].g = g;
					_materials[materialPtr].b = b;
					materialPtr++;
				}
				curNode = curNode.nextSibling;
			}
		}
	}

	//
	private function extractMesh(leNode:XMLNode):Void
	{
		if(_bDispXml == true)
		{
			trace("extractMesh()");
		}
		var curNode:XMLNode = null;
		if(leNode.hasChildNodes())
		{
			curNode = leNode.firstChild;
			while(curNode)
			{
				if(curNode.nodeName == "VERTEXS")
				{
					extractVertexs(curNode, curNode.attributes.amount);
				}
				if(curNode.nodeName == "POLYGONS")
				{
					extractPolygons(curNode, curNode.attributes.amount);
				}
				curNode = curNode.nextSibling;
			}
		}
	}

	//
	private function extractVertexs(leNode:XMLNode, amount:Number):Void
	{
	var	x3d:Number;
	var	y3d:Number;
	var	z3d:Number;
		
		if(_bDispXml == true)
		{
			trace("extractVertexs(" + amount + ")");
		}
		_meshObj.initVertexList(amount);

		var vertexPtr:Number = 0;
		var curNode:XMLNode = null;
		if(leNode.hasChildNodes())
		{
			curNode = leNode.firstChild;
			while(curNode)
			{
				if(curNode.nodeName == "VERTEX")
				{
					x3d = parseFloat(curNode.attributes.x);
					y3d = parseFloat(curNode.attributes.y);
					z3d = parseFloat(curNode.attributes.z);
					_meshObj.setVertex(vertexPtr++, x3d, y3d, z3d);
				}
				curNode = curNode.nextSibling;
			}
		}
	}

	//
	private function extractPolygons(leNode:XMLNode, amount:Number):Void
	{
	var	a:Number;
	var	r:Number;
	var	g:Number;
	var	b:Number;
	var	nx:Number;
	var	ny:Number;
	var	nz:Number;
	var	color:Number;
	var	vert1:Number;
	var	vert2:Number;
	var	vert3:Number;
	var	vert4:Number;
	var	vertArray:Array;
	var	materialNum:Number;

		if(_bDispXml == true)
		{
			trace("extractPolygons(" + amount + ")");
		}
		_meshObj.initPolygonList(amount);

		var polygonPtr:Number = 0;
		var curNode:XMLNode = null;
		if(leNode.hasChildNodes())
		{
			curNode = leNode.firstChild;
			while(curNode)
			{
				if(curNode.nodeName == "POLYGON")
				{
					vertArray = String(curNode.attributes.indexs).split(',');
					vert1 = parseInt(vertArray[0]);
					vert2 = parseInt(vertArray[1]);
					vert3 = parseInt(vertArray[2]);
					vert4 = -1;
					nx = parseFloat(curNode.attributes.nx);
					ny = parseFloat(curNode.attributes.ny);
					nz = parseFloat(curNode.attributes.nz);
					
					materialNum = parseInt(curNode.attributes.material);
					a = _materials[materialNum].a;
					r = _materials[materialNum].r;
					g = _materials[materialNum].g;
					b = _materials[materialNum].b;					
					color = (a << 24) | (r << 16) | (g << 8) | b;
					_meshObj.setPolygon(polygonPtr++, color, vert1, vert2, vert3, vert4, nx, ny, nz);
				}
				curNode = curNode.nextSibling;
			}
		}
	}
}



















