﻿
// Classe transformEngine (V1).
// F Heintz (www.fredHeintz.com)

// Transforms Mesh geometrie.
// Will manage a matrix stack,
// Will manage clipping as well,
// but for now just manage the minimum.

import _engine3DAS.mesh
import _engine3DAS.vertex
import _engine3DAS.polygon
import _engine3DAS.matrix4X4

class _engine3DAS.transformEngine
{
private var	_viewMatrix:matrix4X4;
private var	_projMatrix:matrix4X4;
private var	_worldMatrix:matrix4X4;
private var	_clipMatrix:matrix4X4;
	
	// ----------------------------------------------------------------------------------------------------------------
	// Constructor
	public function transformEngine()
	{
		_worldMatrix = new matrix4X4();
		_viewMatrix = new matrix4X4();
		_projMatrix = new matrix4X4();
		_clipMatrix = new matrix4X4();
	}
	
	// Initialize the world Matrix from 3 Angles.
	public function setWorldMatrix(angleX:Number, angleY:Number, angleZ:Number):Void
	{
		_worldMatrix.setFromAngles(angleX, angleY, angleZ);
	}
	
	// Initialize the view Matrix
	public function setViewMatrix(xView:Number, yView:Number, zView:Number, xTgt:Number, yTgt:Number, zTgt:Number, xUp:Number, yUp:Number, zUp:Number):Void
	{
		_viewMatrix.setLookAt(xView, yView, zView, xTgt, yTgt, zTgt, xUp, yUp, zUp);
	}
	
	// Initialize the projection Matrix
	public function setProjMatrix(fovSinY:Number, fovCosY:Number, aspRatio:Number, nearZ:Number, farZ:Number):Void
	{
		_projMatrix.setIdentity();
		var fovTan:Number = fovCosY / fovSinY;
		_projMatrix._mtrx[0][0] = fovTan / aspRatio;
		_projMatrix._mtrx[1][1] = fovTan;
		_projMatrix._mtrx[2][2] = -(farZ + nearZ) / (farZ - nearZ);
		_projMatrix._mtrx[2][3] = -1.0;
		_projMatrix._mtrx[3][2] = -(farZ * nearZ * 2.0) / (farZ - nearZ);
		_projMatrix._mtrx[3][3] = 0.0;
	}
	
	// Transforms a Mesh vertexList.
	public function transformMesh(meshObj:mesh):Void
	{
	var x3d:Number;
	var y3d:Number;
	var z3d:Number;
	var x2:Number;
	var y2:Number;
	var z2:Number;
	var w2:Number;
	var	r:Number;
	var	g:Number;
	var	b:Number;
	var	Q:Number;
		
		// Build the clipp matrix
		_clipMatrix.mtrxCopy(_projMatrix);
		_clipMatrix.mtrxMultiply(_viewMatrix);
		_clipMatrix.mtrxMultiply(_worldMatrix);

		// Transforms Mesh coordinates
		var vertexObj:vertex;
		for(var i:Number = 0; i < meshObj._nbVertexs; i++)
		{
			vertexObj = meshObj._vertexList[i];
			x3d = vertexObj._x3D;
			y3d = vertexObj._y3D;
			z3d = vertexObj._z3D;
			// Apply transformation
			x2 = (x3d * _clipMatrix._mtrx[0][0]) + (y3d * _clipMatrix._mtrx[1][0]) + (z3d * _clipMatrix._mtrx[2][0]) + _clipMatrix._mtrx[3][0];
			y2 = (x3d * _clipMatrix._mtrx[0][1]) + (y3d * _clipMatrix._mtrx[1][1]) + (z3d * _clipMatrix._mtrx[2][1]) + _clipMatrix._mtrx[3][1];
			z2 = (x3d * _clipMatrix._mtrx[0][2]) + (y3d * _clipMatrix._mtrx[1][2]) + (z3d * _clipMatrix._mtrx[2][2]) + _clipMatrix._mtrx[3][2];
			w2 = (x3d * _clipMatrix._mtrx[0][3]) + (y3d * _clipMatrix._mtrx[1][3]) + (z3d * _clipMatrix._mtrx[2][3]) + _clipMatrix._mtrx[3][3];
			// Perspective divide.
			vertexObj._x2D = ((x2 / w2) + 0.5) * 640.0;
			vertexObj._y2D = ((y2 / w2) + 0.5) * 480.0;
			vertexObj._z = ((z2 + w2) / w2);
			vertexObj._w = w2;
		}
		
		// Compute polygons visibility.
		meshObj.cullMesh();
		
		// Build the lighting matrix
		_clipMatrix.mtrxCopy(_viewMatrix);
		_clipMatrix.mtrxMultiply(_worldMatrix);
		
		// Compute polygon lighting
		var polygonObj:polygon;
		for(var i:Number = 0; i < meshObj._nbPolygons; i++)
		{
			polygonObj = meshObj._polygonList[i];
			if(polygonObj._bVisible != true)
			{
				continue;
			}
			var nx:Number = polygonObj._nx;
			var ny:Number = polygonObj._ny;
			var nz:Number = polygonObj._nz;
			Q = ((nx * _clipMatrix._mtrx[0][2]) + (ny * _clipMatrix._mtrx[1][2]) + (nz * _clipMatrix._mtrx[2][2]));
			polygonObj._nz2 = Q;
			if(Q < 0.0)	Q = -Q;
			r = Math.floor(polygonObj._r * Q);
			g = Math.floor(polygonObj._g * Q);
			b = Math.floor(polygonObj._b * Q);
			polygonObj._color = (r << 16) | (g << 8) | b;			
		}		
	}
}














